#Requires -version 2
[CmdletBinding()]
Param()
Begin{
    $MyPath = Split-Path $MyInvocation.MyCommand.path -Parent
    
    # Load configuration
    $globalConfig = $Config = [xml](Get-Content "$mypath\config.xml")
    $ConnectionString = $globalConfig.Configuration.SQL.ConnectionString
    $Table = $globalConfig.Configuration.SQL.Table
    
    Function Add-FormsObject{
    Param(
        $type,
        $Name,
        $text,
        $Size,
        $Location,
        $tab
    )
        $obj = New-Object System.Windows.Forms.$type
        $obj.Name = $Name
        If($text -ne $null){$obj.text}
        $obj.ClientSize = $Size
        $obj.Location = $Location
        $obj.TabIndex = $tab
    }
    
    Function SaveFileDialog{
    Param(
        [Parameter(Mandatory=$True)]
        $Filetype
    )
        [Void][Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
        $SaveFileDialog = New-Object System.Windows.Forms.SaveFileDialog
        $SaveFileDialog.Filter = "$Filetype files (*.$Filetype)|*.$Filetype|All files (*.*)|*.*"
        $status = $SaveFileDialog.ShowDialog()

        If($status -eq "Cancel"){$Return = $status}
        Else{$Return = $SaveFileDialog.FileName}

        $SaveFileDialog.Dispose()
        Return $Return
    }
    
    Function Call-Form{
        [reflection.assembly]::loadwithpartialname("System.Windows.Forms") | Out-Null
        [reflection.assembly]::loadwithpartialname("System.Drawing") | Out-Null
        $configpath = "$env:localappdata\HotfixBrowser\UserConfig.xml"
        $Config = [xml](Get-Content $configpath)
        
        
        
        $EditMode = $False
        $AdvancedMode = $Config.Root.Config.Advanced.value
        
        ########################################
        ########################################
        # Initialize all objects
        ########################################
        ########################################
        
        # Mainform
        $mainForm = New-Object System.Windows.Forms.Form
        $label7 = New-Object System.Windows.Forms.Label
        $label6 = New-Object System.Windows.Forms.Label
        $NotesSearch = New-Object System.Windows.Forms.TextBox
        $LinkSearch = New-Object System.Windows.Forms.TextBox
        $label5 = New-Object System.Windows.Forms.Label
        $label4 = New-Object System.Windows.Forms.Label
        $Progressbar = New-Object System.Windows.Forms.ProgressBar
        $DownloadSearch = New-Object System.Windows.Forms.ComboBox
        $label3 = New-Object System.Windows.Forms.Label
        $label2 = New-Object System.Windows.Forms.Label
        $label1 = New-Object System.Windows.Forms.Label
        $ReviewSearch = New-Object System.Windows.Forms.TextBox
        $AppliestoSearch = New-Object System.Windows.Forms.TextBox
        $ShortTextSearch = New-Object System.Windows.Forms.TextBox
        $KBIDsearch = New-Object System.Windows.Forms.TextBox
        $searchButton = New-Object System.Windows.Forms.Button
        $dataGrid = New-Object System.Windows.Forms.DataGridView
        $InitialFormWindowState = New-Object System.Windows.Forms.FormWindowState
        $AdvancedQueryLabel = New-Object System.Windows.Forms.Label
        $AdvancedQueryBox = New-Object System.Windows.Forms.TextBox
        
        #Select Column form
        $ColumnForm = New-Object System.Windows.Forms.Form
        $ColOKButton = New-Object System.Windows.Forms.Button
        $listViewCol = New-Object System.Windows.Forms.ListView
        
        #SaveSearch form
        $saveForm = New-Object System.Windows.Forms.Form
        $saveBox = New-Object System.Windows.Forms.TextBox
        $SaveButton = New-Object System.Windows.Forms.Button
        $listViewLoadSave = New-Object System.Windows.Forms.ListView
        
        #LoadSearch form
        $loadForm = New-Object System.Windows.Forms.Form
        $loadBox = New-Object System.Windows.Forms.TextBox
        $loadButton = New-Object System.Windows.Forms.Button
        
        ########################################
        ########################################
        # Button/script area
        ########################################
        ########################################
        
        $MainForm_onLoad = {
            .$SwitchAdvanced
        }
        
        $SwitchAdvanced = {
            $Boxes = Get-childItem variable: | Where-Object{($_.Name -match "search") -and (($_.Value -match ".Textbox") -or ($_.Value -match ".ComboBox"))}
            $Labels = Get-childItem variable: | Where-Object{$_.Name.Startswith("label")}
            If($AdvancedMode -eq $true){
                $fileAdvanced.Checked = $true
                $Boxes | ForEach-Object{(Invoke-Expression "`$$($_.Name)").Visible = $False}
                $Labels | ForEach-Object{(Invoke-Expression "`$$($_.Name)").Visible = $False}
                
                $AdvancedQueryLabel.Visible = $True
                $AdvancedQueryBox.Visible = $True
            }
            Else{
                $fileAdvanced.Checked = $false
                $Boxes | ForEach-Object{(Invoke-Expression "`$$($_.Name)").Visible = $True}
                $Labels | ForEach-Object{(Invoke-Expression "`$$($_.Name)").Visible = $True}
                
                $AdvancedQueryLabel.Visible = $False
                $AdvancedQueryBox.Visible = $False
            }
        }
        
        $searchButton_Click ={
            $Progressbar.Show()
            If($DownloadSearch.Text -eq "True"){
                $DL = "1"
            }
            Else{
                $DL = "0"
            }
            
            If($AdvancedMode -eq $false){
                #Get values from fields
                $SearchValues = @{}
                $SearchValues.Add("KBID",$KBIDsearch.Text)
                $SearchValues.Add("ShortText",$ShortTextSearch.Text)
                $SearchValues.Add("Download",$DL)
                $SearchValues.Add("AppliesTo",$AppliestoSearch.Text)
                $SearchValues.Add("LastReviewed",$ReviewSearch.Text)
                $SearchValues.Add("Link",$LinkSearch.Text)
                $SearchValues.Add("Notes",$NotesSearch.Text)
                
                #Compile search query
                $Query = "Select * FROM $table WHERE Exist = 1"
                $SearchValues.Keys | ForEach-Object{
                    If($SearchValues.$_ -ne ''){
                        $value = $SearchValues.$_
                        $Query = "$Query AND $_ LIKE `'$Value`'"
                    }
                    
                }
            }
            Else{
                $Query = $AdvancedQueryBox.Text
            }
            .$RunQuery
            
            $Progressbar.Hide()
        }
        
        # Menu button clicks
        
        $toolEdit_Click = {
            If($EditMode -eq $False){
                $toolEdit.Checked = $True
                $EditMode = $True
                $MainMenu.Items.Add($editMenu) | out-null
                $datagrid.columns | ForEach-Object{$_.ReadOnly = $False}
            }
            Else{
                $toolEdit.Checked = $False
                $EditMode = $False
                $MainMenu.Items.Remove($editMenu) | out-null
                $datagrid.columns | ForEach-Object{$_.ReadOnly = $True}
            }
        }
        
        $fileExport_Click = {
            $File = SaveFileDialog -Filetype "csv"
            If($File -ne "Cancel"){
                $Progressbar.Show()
                "KBID`tShortText`tDownload`tAppliesTo`tLastReviewed`tLink`tNotes" | Out-File $File
                $dataGrid.Rows | ForEach-Object{
                    $Cells = $_.cells
                    $KBID = ($cells | Where-object{$_.OwningColumn.Name -eq "KBID"}).Value
                    $ShortText = ($cells | Where-object{$_.OwningColumn.Name -eq "ShortText"}).Value
                    $Download = ($cells | Where-object{$_.OwningColumn.Name -eq "Download"}).Value
                    $AppliesTo = ($cells | Where-object{$_.OwningColumn.Name -eq "AppliesTo"}).Value
                    $LastReviewed = ($cells | Where-object{$_.OwningColumn.Name -eq "LastReviewed"}).Value
                    $Link = ($cells | Where-object{$_.OwningColumn.Name -eq "Link"}).Value
                    $Notes = ($cells | Where-object{$_.OwningColumn.Name -eq "Notes"}).Value
                    If($KBID -ne $null){
                        "$KBID`t$ShortText`t$Download`t$AppliesTo`t$LastReviewed`t$Link`t$Notes" | Out-File $File -Append
                    }                    
                }
                $Progressbar.Hide()
            }
        }
        
        $toolClear_Click = {
            $dataGrid.Rows.Clear()
            $KBIDsearch.Text = ''
            $ShortTextSearch.Text = ''
            $DownloadSearch.Text = "True"
            $AppliestoSearch.Text = ''
            $ReviewSearch.Text = ''
            $LinkSearch.Text = ''
            $NotesSearch.Text = ''
        }
        
        $toolSave_Click = {
            .$saveloadOnLoad
            $saveForm.Controls.Add($listViewLoadSave)
            $saveForm.ShowDialog() | Out-Null
        }
        $toolLoad_Click = {
            .$saveloadOnLoad
            $loadForm.Controls.Add($listViewLoadSave)
            $loadForm.ShowDialog() | Out-Null
        }
        
        $toolColumns_Click = {
            .$ColumnsOnLoad
            $ColumnForm.ShowDialog() | Out-Null
        }
        
        $fileAdvanced_Clicked = {
            If($AdvancedMode -eq $true){
                $Config.Root.Config.Advanced.value = "False"
                
            }
            Else{
                $Config.Root.Config.Advanced.value = "True"
            }
            $Config.Save($configpath)
            $Config = [xml](Get-Content $configpath)
            $AdvancedMode = $Config.Root.Config.Advanced.value
            .$SwitchAdvanced
        }
        
        $fileQuit_Click = {
            $mainForm.Close()
        }
        
        $ColOKButton_OnClick = {
            $ColumnForm.Close()
            $ToHide = $listViewCol.Items | Where-Object{$_.Checked -eq $False} | Select -ExpandProperty Text
            
            Try{$Config.Root.Config.Columns.Hide | ForEach-Object{$_.RemoveAll()}}Catch{}
            $Config.Save($configpath)
            
            $ToHide | ForEach-Object{
                $Name = $_
                $new = $Config.CreateElement('Hide')
                $new.SetAttribute('Column',"$Name")
                $Config.Root.Config.Columns.AppendChild($new)
                $Config.Save($configpath)
            }
            .$RefreshList
        }
        
        $RefreshList = {
            
            #clean up bad elements
            [xml]$Clean = Get-Content $configpath | Where-Object{($_ -NotMatch '<Hide />') -and ($_ -NotMatch 'Hide Column=""')}
            $Clean.Save($configpath)
            
            $ToHide = $Clean.Root.Config.Columns.Hide | Select -ExpandProperty Column
            ($dataGrid.Columns | Where-Object{$ToHide -Contains "$($_.Name)"}) | ForEach-Object{$_.Visible = $False}
            ($dataGrid.Columns | Where-Object{$ToHide -NotContains "$($_.Name)"}) | ForEach-Object{$_.Visible = $True}
            
            $Count = $dataGrid.RowCount
            0..$Count | ForEach-Object{
                Try{$dataGrid.Rows[$_].HeaderCell.Value = "$($_+1)"}
                Catch{}
            }
        }
        
        $SaveButton_OnClick = {
            $new = $Config.CreateElement('Save')
            $Name = $SaveBox.Text
            $new.SetAttribute('Name',"$($SaveBox.Text)")
            $new.SetAttribute('Query',"$Query")
            
            Try{($Config.Root.SavedSearches.Save | Where-Object{$_.Name -eq $Name}).RemoveAll()}Catch{}
            $Config.Root.SavedSearches.AppendChild($new)
            $Config.Save($configpath)
            
            #clean up bad elements
            [xml]$Clean = Get-Content $configpath | Where-Object{($_ -NotMatch '<Save />') -and ($_ -NotMatch 'Save Name=""')}
            $Clean.Save($configpath)
            $saveForm.Controls.Remove($listViewLoadSave)
            $saveForm.Close()
        }
        $LoadButton_OnClick = {
            $Query = ($Config.Root.SavedSearches.Save | Where-object {$_.Name -Match "$($loadBox.Text)"}).Query
            $loadForm.Controls.Remove($listViewLoadSave)
            $loadForm.Close()
            $Progressbar.Show()
            $KBIDsearch.Text = ''
            $ShortTextSearch.Text = ''
            $DownloadSearch.Text = "True"
            $AppliestoSearch.Text = ''
            $ReviewSearch.Text = ''
            $LinkSearch.Text = ''
            $NotesSearch.Text = ''
            .$RunQuery
            $Progressbar.Hide()
        }
        
        $RunQuery = {
            $AdvancedQueryBox.Text = $Query
            Try{
                $Data = Get-SQLQuery $Query
                $dataGrid.Rows.Clear()
                }
            Catch{}
            
            #Get Col headers
            $Columns = $Script:DataColumns | Select -ExpandProperty Caption
            #Clear old headers
            While($DataGrid.Columns.Count -gt 0){
                Try{$DataGrid.Columns | ForEach-Object{$DataGrid.Columns.Remove($_)}}
                Catch{}
            }
            
            #Populate headers
            $Columns | ForEach-Object{
                $Column = New-Object System.Windows.Forms.DataGridViewTextBoxColumn
                $Column.HeaderText = "$_"
                $Column.Name = "$_"
                $Column.ReadOnly = $True
                $Column.Width = 100

                $dataGrid.Columns.Add($Column)|Out-Null
            }
            
            $Data | ForEach-Object{
                $Row = $_
                $thisrow = $Columns | ForEach-Object{
                    $CurrentCol = $_
                    # Format datetime to sortable
                    Try{
                        $date = [DateTime]$row.$CurrentCol
                        '{0:yyyy-MM-dd}' -f $Date
                    }
                    # Output everything else as is
                    Catch{
                        $row.$CurrentCol
                    }
                }
                $dataGrid.Rows.Add($thisrow)
            }
            .$RefreshList
        }
        
        $listViewLoadSave_OnSelected = {
            $SaveBox.Text = $listViewLoadSave.SelectedItems[0].Text
            $loadBox.Text = $listViewLoadSave.SelectedItems[0].Text
        }
        
        $listViewLoadSave_DoubleClick = {
            If($SaveButton.Visible -eq $True){.$SaveButton_OnClick}
            Else{.$LoadButton_OnClick}
        }
        
        $saveloadOnLoad = {
            $loadBox.Text = ''
            $SaveBox.Text = ''
            $listViewLoadSave.Items.Clear()
            [xml]$Clean = Get-Content $configpath | Where-Object{($_ -NotMatch '<Save />') -and ($_ -NotMatch 'Save Name=""')}
            $Clean.Save($configpath)
            $Config = [xml](Get-Content $configpath)
            $Config.Root.SavedSearches.Save | ForEach-Object{
                $listViewLoadSave.Items.Add("$($_.Name)") | Out-Null
            }
        }
        
        $ColumnsOnLoad = {
            $listViewCol.Items.Clear()
            $Columns = $Script:DataColumns | Select -ExpandProperty Caption
            $Columns | ForEach-Object{
                $listViewCol.Items.Add("$_") | Out-Null
            }
            $Config = [xml](Get-Content $configpath)
            $ToHide = $Config.Root.Config.Columns.Hide | Select -ExpandProperty Column
            $ListViewCol.Items | ForEach-Object{
                If($ToHide -NotContains ($_.Text)){
                    $_.Checked = $True
                }
            }
        }
        
        $editClear_Click = {
            If(!(Test-path -Path "$env:localappdata\HotfixBrowser" -PathType Container)){New-Item -Path "$env:localappdata\HotfixBrowser" -ItemType Container}
            $new = "<?xml version=`"1.0`" encoding=`"utf-8`"?><Root><SavedSearches Description=`"All saved searches are nodes under this one`"></SavedSearches><Config Description=`"General configuration`"><Columns Description=`"Config for hidden columns`"></Columns><Advanced value=`"False`" /></Config></Root>"
            $new | out-file $configpath
            $Config = [xml](Get-Content $configpath) 
        }
        
        $editUpdate_Click = {
        
            $dataGrid.Rows | ForEach-Object{
                $Cells = $_.cells
                
                $count = $cells.count -1
                $cols = @()
                $values = @()
                
                0..$count | ForEach-Object{
                    $cols += $columns[$_]
                    
                    If(($cells[$_].Value).Length -gt 0){
                        $values += "`'$($cells[$_].Value)`'"
                    }Else{
                        $values += "NULL"
                    }
                }
                
                $UpdateCMD = "UPDATE $table SET $($c = $cols.count;(1..($c-1) | ForEach-Object{"$($cols[$_]) = $($Values[$_])"}) -Join ',') WHERE KBID = `'$($cells[0].Value)`';"
                Process-SQLCmd $UpdateCMD
            }
        }
        
        ########################################
        ########################################
        # Form settings area
        ########################################
        ########################################

        $mainForm.ClientSize = "853,485"
        $mainForm.DataBindings.DefaultDataSourceUpdateMode = 0
        $mainForm.Text = "KB article browser"
        ### Main Menu Settings
        $MainMenu = new-object System.Windows.Forms.MenuStrip
        $MainMenu.Location = "0,0"
        $MainMenu.Name = "MainMenu"
        $MainMenu.Size = "1000,620"
        $MainMenu.TabIndex = 0
        $MainMenu.Text = "Main Menu"
        
        ## File Menu settings
        $fileMenu = new-object System.Windows.Forms.ToolStripMenuItem('&fileMenu')
        $fileMenu.Size = "35,20"
        $fileMenu.Text = "File"
        
        # Save button
        $fileExport = new-object System.Windows.Forms.ToolStripMenuItem('&Export list to CSV')
        $fileExport.Add_Click($fileExport_Click)
        
        # Save button
        $fileAdvanced = new-object System.Windows.Forms.ToolStripMenuItem('&Advanced mode')
        $fileAdvanced.Add_Click($fileAdvanced_Clicked)
        
        # Quit button
        $fileQuit = new-object System.Windows.Forms.ToolStripMenuItem('&Quit')
        $fileQuit.add_Click($fileQuit_Click)
        
        # Add to file menu
        $fileMenu.DropDownItems.Add($fileAdvanced) | out-null
        $fileMenu.DropDownItems.Add($fileExport) | out-null
        $fileMenu.DropDownItems.Add($fileQuit) | out-null
        
        ## Tool Menu settings
        $toolMenu = new-object System.Windows.Forms.ToolStripMenuItem('&toolMenu')
        $toolMenu.Size = "35,20"
        $toolMenu.Text = "Tools"
        
        # Save button
        $toolSave = new-object System.Windows.Forms.ToolStripMenuItem('&Save search')
        $toolSave.add_Click($toolSave_Click)
        
        # Load button
        $toolLoad = new-object System.Windows.Forms.ToolStripMenuItem('&Load search')
        $toolLoad.add_Click($toolLoad_Click)
        
        # Columns button
        $toolColumns = new-object System.Windows.Forms.ToolStripMenuItem('&Select columns')
        $toolColumns.add_Click($toolColumns_Click)
        
        # Edit button
        $toolEdit = new-object System.Windows.Forms.ToolStripMenuItem('&Edit mode')
        $toolEdit.add_Click($toolEdit_Click)

        # Clear button
        $toolClear = new-object System.Windows.Forms.ToolStripMenuItem('&Clear/Reset')
        $toolClear.add_Click($toolClear_Click)
         
        # Add to tool menu
        $toolMenu.DropDownItems.Add($toolSave) | out-null
        $toolMenu.DropDownItems.Add($toolLoad) | out-null
        $toolMenu.DropDownItems.Add($toolColumns) | out-null
        $toolMenu.DropDownItems.Add($toolEdit) | out-null
        $toolMenu.DropDownItems.Add($toolClear) | out-null
        
        ## Edit Menu settings
        $editMenu = new-object System.Windows.Forms.ToolStripMenuItem('&editMenu')
        $editMenu.Size = "35,20"
        $editMenu.Text = "Edit"
        
        # Clear saved searches button
        $editClear = new-object System.Windows.Forms.ToolStripMenuItem('&Clear all settings')
        $editClear.add_Click($editClear_Click)
        
        # Update button
        $editUpdate = new-object System.Windows.Forms.ToolStripMenuItem('&Update database')
        $editUpdate.add_Click($editUpdate_Click)
        
        # Add to tool menu
        $editMenu.DropDownItems.Add($editUpdate) | out-null
        $editMenu.DropDownItems.Add($editClear) | out-null
        
        # Add main menu buttons
        $MainMenu.Items.Add($fileMenu) | out-null
        $MainMenu.Items.Add($toolMenu) | out-null
        
        $mainForm.Controls.Add($MainMenu) | out-null
        
        #######AdvancedView#########
        
        $AdvancedQueryLabel.Location = "10,43"
        $AdvancedQueryBox.Anchor = "15"
        $AdvancedQueryLabel.Size = "300,14"
        $AdvancedQueryLabel.Text = "Input SQL query here"
        $AdvancedQueryLabel.Visible = $false
        
        $mainForm.Controls.Add($AdvancedQueryLabel) | out-null
        
        $AdvancedQueryBox.Location = "10,64"
        $AdvancedQueryBox.Anchor = "15"
        $AdvancedQueryBox.Size = "720,20"
        $AdvancedQueryBox.Visible = $false
        $mainForm.Controls.Add($AdvancedQueryBox) | out-null
        
        #######Searchboxes##########
        
        $NotesSearch.DataBindings.DefaultDataSourceUpdateMode = 0
        $NotesSearch.Location = "653,64"
        $NotesSearch.Name = "NotesSearch"
        $NotesSearch.Size = "93,20"
        $NotesSearch.TabIndex = 15

        $mainForm.Controls.Add($NotesSearch)

        $LinkSearch.DataBindings.DefaultDataSourceUpdateMode = 0
        $LinkSearch.Location = "553,64"
        $LinkSearch.Name = "LinkSearch"
        $LinkSearch.Size = "94,20"
        $LinkSearch.TabIndex = 14

        $mainForm.Controls.Add($LinkSearch)
        
        $DownloadSearch.DataBindings.DefaultDataSourceUpdateMode = 0
        $DownloadSearch.FormattingEnabled = $False
        $DownloadSearch.Items.Add("True")|Out-Null
        $DownloadSearch.Items.Add("False")|Out-Null
        $DownloadSearch.Location = "253,64"
        $DownloadSearch.Name = "DownloadSearch"
        $DownloadSearch.Size = "94,21"
        $DownloadSearch.TabIndex = 10
        $DownloadSearch.SelectedItem = "True"
        $DownloadSearch.DropDownStyle = 2
        
        $mainForm.Controls.Add($DownloadSearch)
        
        $ReviewSearch.DataBindings.DefaultDataSourceUpdateMode = 0
        $ReviewSearch.Location = "453,65"
        $ReviewSearch.Name = "ReviewSearch"
        $ReviewSearch.Size = "94,20"
        $ReviewSearch.TabIndex = 6

        $mainForm.Controls.Add($ReviewSearch)

        $AppliestoSearch.DataBindings.DefaultDataSourceUpdateMode = 0
        $AppliestoSearch.Location = "353,65"
        $AppliestoSearch.Name = "AppliestoSearch"
        $AppliestoSearch.Size = "94,20"
        $AppliestoSearch.TabIndex = 5

        $mainForm.Controls.Add($AppliestoSearch)

        $ShortTextSearch.DataBindings.DefaultDataSourceUpdateMode = 0
        $ShortTextSearch.Location = "153,65"
        $ShortTextSearch.MaxLength = 128
        $ShortTextSearch.Name = "ShortTextSearch"
        $ShortTextSearch.Size = "94,20"
        $ShortTextSearch.TabIndex = 3

        $mainForm.Controls.Add($ShortTextSearch)

        $KBIDsearch.DataBindings.DefaultDataSourceUpdateMode = 0
        $KBIDsearch.Location = "53,65"
        $KBIDsearch.MaxLength = 7
        $KBIDsearch.Name = "KBIDsearch"
        $KBIDsearch.Size = "94,20"
        $KBIDsearch.TabIndex = 2

        $mainForm.Controls.Add($KBIDsearch)
        
        $searchButton.Anchor = 9

        $searchButton.DataBindings.DefaultDataSourceUpdateMode = 0
        $searchButton.Location = "752,64"
        $searchButton.Name = "searchButton"
        $searchButton.Size = "88,21"
        $searchButton.TabIndex = 1
        $searchButton.Text = "Search"
        $searchButton.UseVisualStyleBackColor = $True
        $searchButton.add_Click($searchButton_Click)

        $mainForm.Controls.Add($searchButton)
        
        #####Labels and the rest#####

        $label7.DataBindings.DefaultDataSourceUpdateMode = 0

        $label7.Location = "653,43"
        $label7.Name = "label7"
        $label7.Size = "100,15"
        $label7.TabIndex = 17
        $label7.Text = "Notes"

        $mainForm.Controls.Add($label7)

        $label6.DataBindings.DefaultDataSourceUpdateMode = 0

        $label6.Location = "554,43"
        $label6.Name = "label6"
        $label6.Size = "100,14"
        $label6.TabIndex = 16
        $label6.Text = "Link"

        $mainForm.Controls.Add($label6)

        $label5.DataBindings.DefaultDataSourceUpdateMode = 0

        $label5.Location = "454,43"
        $label5.Name = "label5"
        $label5.Size = "93,18"
        $label5.TabIndex = 13
        $label5.Text = "Last Review Date"

        $mainForm.Controls.Add($label5)

        $label4.DataBindings.DefaultDataSourceUpdateMode = 0

        $label4.Location = "353,43"
        $label4.Name = "label4"
        $label4.Size = "94,15"
        $label4.TabIndex = 12
        $label4.Text = "AppliesTo"

        $mainForm.Controls.Add($label4)

        $Progressbar.DataBindings.DefaultDataSourceUpdateMode = 0
        $Progressbar.Location = "13,437"
        $Progressbar.Name = "Progressbar"
        $Progressbar.Size = "828,36"
        $Progressbar.Style = 2
        $Progressbar.TabIndex = 11
        $Progressbar.Anchor = 14

        $mainForm.Controls.Add($Progressbar)
        $Progressbar.Hide()

        $label3.DataBindings.DefaultDataSourceUpdateMode = 0

        $label3.Location = "253,43"
        $label3.Name = "label3"
        $label3.Size = "94,18"
        $label3.TabIndex = 9
        $label3.Text = "Hotfix available"

        $mainForm.Controls.Add($label3)

        $label2.DataBindings.DefaultDataSourceUpdateMode = 0

        $label2.Location = "153,44"
        $label2.Name = "label2"
        $label2.Size = "94,18"
        $label2.TabIndex = 8
        $label2.Text = "Short Text"

        $mainForm.Controls.Add($label2)

        $label1.DataBindings.DefaultDataSourceUpdateMode = 0
        $label1.Location = "53,44"
        $label1.Name = "label1"
        $label1.RightToLeft = 0
        $label1.Size = "94,18"
        $label1.TabIndex = 7
        $label1.Text = "KB ID"
        $label1.add_Click($handler_label1_Click)

        $mainForm.Controls.Add($label1)
        
        $dataGrid.DataBindings.DefaultDataSourceUpdateMode = 0
        $dataGrid.Anchor = "15"
        $dataGrid.Location = "12,91"
        $dataGrid.Name = "dataGrid"
        $dataGrid.Size = "828,382"
        $dataGrid.TabIndex = 0
        
        $mainForm.Controls.Add($dataGrid)

        ### SELECT COLUMN FORM SETTINGS ###
        
        $ColumnForm.ClientSize = "246,196"
        $ColumnForm.DataBindings.DefaultDataSourceUpdateMode = 0
        $ColumnForm.Name = "ColumnForm"
        $ColumnForm.Text = "Select columns"
        $ColumnForm.add_Load($ColumnsOnLoad)
        
        $ColOKButton.Anchor = 14
        $ColOKButton.DataBindings.DefaultDataSourceUpdateMode = 0
        $ColOKButton.Location = "1,159"
        $ColOKButton.Name = "ColOKButton"
        $ColOKButton.Size = "245,38"
        $ColOKButton.TabIndex = 1
        $ColOKButton.Text = "OK"
        $ColOKButton.UseVisualStyleBackColor = $True
        $ColOKButton.add_Click($ColOKButton_OnClick)
        $ColumnForm.Controls.Add($ColOKButton)
        
        $listViewCol.Anchor = 15
        $listViewCol.DataBindings.DefaultDataSourceUpdateMode = 0
        $listViewCol.Location = "1,1"
        $listViewCol.Name = "listView"
        $listViewCol.Size = "245,155"
        $listViewCol.TabIndex = 0
        #$listViewCol.add_MouseClick($listViewLoadSave_OnSelected)
        #$listViewCol.add_DoubleClick($listViewLoadSave_DoubleClick)
        $listViewCol.View = 3
        $listViewCol.CheckBoxes = $True
        $ColumnForm.Controls.Add($listViewCol)
        
        ### SAVE SEARCH FORM SETTINGS
        $saveForm.ClientSize = "246,196"
        $saveForm.DataBindings.DefaultDataSourceUpdateMode = 0
        $saveForm.Name = "saveForm"
        $saveForm.Text = "Save current search"
        $saveForm.add_Load($saveloadOnLoad)

        $saveBox.Anchor = 14
        $saveBox.DataBindings.DefaultDataSourceUpdateMode = 0
        $saveBox.Location = "1,133"
        $saveBox.Name = "saveBox"
        $saveBox.Size = "245,20"
        $saveBox.TabIndex = 2

        $saveForm.Controls.Add($saveBox)

        $SaveButton.Anchor = 14
        $SaveButton.DataBindings.DefaultDataSourceUpdateMode = 0
        $SaveButton.Location = "1,159"
        $SaveButton.Name = "SaveButton"
        $SaveButton.Size = "245,38"
        $SaveButton.TabIndex = 1
        $SaveButton.Text = "Save"
        $SaveButton.UseVisualStyleBackColor = $True
        $SaveButton.add_Click($SaveButton_OnClick)

        $saveForm.Controls.Add($SaveButton)

        $listViewLoadSave.Anchor = 15
        $listViewLoadSave.DataBindings.DefaultDataSourceUpdateMode = 0
        $listViewLoadSave.Location = "1,1"
        $listViewLoadSave.Name = "listView"
        $listViewLoadSave.Size = "245,121"
        $listViewLoadSave.TabIndex = 0
        $listViewLoadSave.add_MouseClick($listViewLoadSave_OnSelected)
        $listViewLoadSave.add_DoubleClick($listViewLoadSave_DoubleClick)
        $listViewLoadSave.View = 3
        
        ### LOAD SEARCH FORM SETTINGS
        $loadForm.ClientSize = "246,196"
        $loadForm.DataBindings.DefaultDataSourceUpdateMode = 0
        $loadForm.Name = "loadForm"
        $loadForm.Text = "Load current search"
        $loadForm.add_Load($saveloadOnLoad)

        $loadBox.Anchor = 14
        $loadBox.DataBindings.DefaultDataSourceUpdateMode = 0
        $loadBox.Location = "1,133"
        $loadBox.Name = "loadBox"
        $loadBox.Size = "245,20"
        $loadBox.TabIndex = 2

        $loadForm.Controls.Add($loadBox)

        $loadButton.Anchor = 14
        $loadButton.DataBindings.DefaultDataSourceUpdateMode = 0
        $loadButton.Location = "1,159"
        $loadButton.Name = "loadButton"
        $loadButton.Size = "245,38"
        $loadButton.TabIndex = 1
        $loadButton.Text = "load"
        $loadButton.UseVisualStyleBackColor = $True
        $loadButton.add_Click($loadButton_OnClick)
        
        $loadForm.Controls.Add($listViewLoadSave)
        $loadForm.Controls.Add($loadButton)
        
        #Show the Form
        If(!(Test-Path $configpath)){.$editClear_Click}
        $mainForm.add_Load($MainForm_onLoad)
        $mainForm.ShowDialog()| Out-Null
    }
    
    Function Process-SQLCmd {
    Param(
        [Parameter(Mandatory=$true)]
        [String]$Command,
        [Parameter(Mandatory=$false)]
        [String]$SQLConnection = $ConnectionString
    )
        $connection = New-Object System.Data.SqlClient.SqlConnection
        $connection.ConnectionString = $SQLConnection
        $connection.Open()
        $cmd = New-Object System.Data.SQLClient.SQLCommand
        $cmd.CommandTimeout = 300
        $cmd.Connection = $connection
        $cmd.CommandText = $Command
        Try{$cmd.ExecuteNonQuery()}
        Catch{
            If(!($_.Exception -match 'Violation of PRIMARY KEY constraint')){
                $_.Exception | Out-file "$Mypath\error.log" -append
                $command | Out-file "$Mypath\error.log" -append
            }
        }
        $connection.Close()
    }
    
    Function Get-SQLQuery{
    Param(
        [Parameter(Mandatory=$true)]
        [String]$SQLQuery,
        [Parameter(Mandatory=$false)]
        [String]$SQLConnection = $ConnectionString
    )
        $connection = New-Object System.Data.SqlClient.SqlConnection
        $connection.ConnectionString = $SQLConnection
        $connection.Open()
        $Command = New-Object System.Data.SQLClient.SQLCommand
        $Command.CommandTimeout = 300
        $Command.Connection = $Connection
        $Command.CommandText = $SQLQuery
        Try{
            $Reader = $Command.ExecuteReader()
            $Counter = $Reader.FieldCount
            
            $DataTable = New-Object system.Data.DataTable
            for($i = 0; $i -lt $counter; $i++){
                $Column = New-Object system.Data.DataColumn $Reader.GetName($i),([string])
                $DataTable.Columns.Add($Column)
            }
            
            while ($Reader.Read()) {
                $strData = @()
                for ($i = 0; $i -lt $Counter; $i++) {
                    $strData += [String]$Reader.GetValue($i)
                }
                $DataTable.Rows.Add($strData) | out-null
            }
            $Reader.Close()
            $Script:DataColumns = $DataTable.Columns
        }
        Catch{
            Write-Error $_
            $Connection.Close()
            Return ($_.Exception)
        }
        Finally{
            $Connection.Close()
        }
        Return $Datatable
    }
}
Process{
    
    Call-Form
}
End{}